import tkinter
import math
import time
from random import randint

def calculateDistance( clasterX, clasterY, pointX, pointY ):
    vectX = clasterX - pointX
    vectY = clasterY - pointY
    return math.sqrt( math.pow(vectX, 2) + math.pow(vectY, 2) )

def runKmeans( canvasVisual, numPoints, numClasters ):
    
    # Developer ver. print everything
    print( "\n--> Started generating points: " + numPoints + " points" )
    
    # Clear canvas
    KmeansGui.clearCanvas( canvasVisual )
    
    # Define array ( Points )
    pointsArray = [[0] * 4 for i in range( int( numPoints ) )]
    
    # Fill array ( Points )
    for i in range( int( numPoints ) ):
        pointsArray[i][1] = randint(0, 799 )
        pointsArray[i][2] = randint( 0,  799 )
        pointsArray[i][3] = 99999
        print( "\n * Created point: " + str( i ) + " X: " + str( pointsArray[i][1] ) + " Y: " + str( pointsArray[i][2] ) )
        # Create visual points
        KmeansGui.createPoint( canvasVisual, int( pointsArray[i][1] ), int( pointsArray[i][2] ), 6, "green" )

    print("\n--> Points generated and displayed\n\n--> Started generating clusters")

    # Define array ( Clasters )
    clastersArray = [[0] * 5 for i in range( int( numClasters ) )]

    # Fill array ( Clasters )
    for i in range( int( numClasters ) ):
        clastersArray[i][1] = clastersArray[i][3] = randint( 0, 799 )
        clastersArray[i][2] = clastersArray[i][4] = randint( 0, 799 )
        print( "\n * Created cluster: " + str( i ) + " X: " + str( clastersArray[i][1] ) + " Y: " + str( clastersArray[i][2] ) )
        # Create visual clasters
        KmeansGui.createPoint( canvasVisual, int( clastersArray[i][1] ), int( clastersArray[i][2] ), 12, "blue" )

    startTime = time.time()
    clastersStay = 1
    iterations = 0

    while (clastersStay != 0):
        if ( startTime - time.time() < -2 ):

            startTime = time.time()
            clastersStay = 0
            iterations += 1

            # Return lenght from k punct
            for i in range( int( numClasters ) ):
                for j in range( int( numPoints ) ):
                    distance = calculateDistance( clastersArray[i][1], clastersArray[i][2], pointsArray[j][1], pointsArray[j][2] )
                    if distance < pointsArray[j][3]:
                        pointsArray[j][0] = i
                        clastersArray[i][0] = j
                        pointsArray[j][3] = distance

            # Clear canvas
            KmeansGui.clearCanvas( canvasVisual )

            print( "\n--> Making graphic" )

            # Create visual lines
            for i in range( int( numClasters ) ):
                KmeansGui.createPoint( canvasVisual, int( clastersArray[i][1] ), int( clastersArray[i][2] ), 12, "blue" )
                for j in range( int( numPoints ) ):
                    KmeansGui.createPoint( canvasVisual, int( pointsArray[j][1] ), int( pointsArray[j][2] ), 6, "green" )
                    if pointsArray[j][0] == i:
                        KmeansGui.createLine( canvasVisual, clastersArray[i][1], clastersArray[i][2], pointsArray[j][1], pointsArray[j][2], "black" )

            for i in range( int( numClasters ) ):
                # Define needed variables
                clasterNewX = 0
                clasterNewY = 0
                pointsCounter = 0
                for j in range( int( numPoints ) ):
                    if pointsArray[j][0] == i:
                        clasterNewX += pointsArray[j][1] 
                        clasterNewY += pointsArray[j][2]
                        pointsCounter += 1
                if pointsCounter != 0:
                    clastersArray[i][3] = clastersArray[i][1]
                    clastersArray[i][4] = clastersArray[i][2]
                    clastersArray[i][1] = clasterNewX/pointsCounter
                    clastersArray[i][2] = clasterNewY/pointsCounter

            for i in range( int( numClasters ) ):
                clastersStay += ( clastersArray[i][1] - clastersArray[i][3] ) + ( clastersArray[i][2] - clastersArray[i][4] )

    print( "\n * Ilosc powtorzen: " + str( iterations ) )
    
    print( "\n--> Koniec" )

class KmeansGui:

    def __init__(self, master):
        self.master = master
        master.title("K-means graphic")
        master.geometry("1010x850")

        # Settings in this frame
        self.settingsFrame = tkinter.Frame( master )
        self.settingsFrame.pack( side = "top" )
        # Visual data in this frame
        self.canvasFrame = tkinter.Frame( master )
        self.canvasFrame.pack( side = "bottom" )

        # Label and Entry - how many points
        self.labelPoints = tkinter.Label( self.settingsFrame, text = "  Liczba danych " )
        self.labelPoints.pack( side = "left" )

        self.entryPoints = tkinter.Entry( self.settingsFrame, bd = 1 )
        self.entryPoints.pack( side = "left" )

        # Label and Entry - how many K clasters
        self.labelClasters = tkinter.Label( self.settingsFrame, text = "  Liczba punktow K " )
        self.labelClasters.pack( side = "left" )

        self.entryClasters = tkinter.Entry( self.settingsFrame, bd = 1 )
        self.entryClasters.pack( side = "left" )

        self.startButton = tkinter.Button( self.settingsFrame, text="  Start  ", command=self.startAlgorithm )
        self.startButton.pack( side = "right" )

        # Canvas
        self.canvasVisual = tkinter.Canvas( self.canvasFrame, width = 800, height = 800, bg = "white", bd = 0 )
        self.canvasVisual.pack()

    def startAlgorithm(self):
        runKmeans( self.canvasVisual, self.entryPoints.get(), self.entryClasters.get() )

    def clearCanvas( canvasVisual ):
        canvasVisual.delete("all")

    def createPoint( canvasVisual, xPos, yPos, size, fillColor ):
        canvasVisual.create_oval( xPos, yPos, ( xPos + size ), ( yPos + size ), fill = fillColor, outline = "" )
        canvasVisual.update()

    def createLine( canvasVisual, xPos, yPos, xPos2, yPos2, fillColor ):
        canvasVisual.create_line( xPos, yPos, xPos2, yPos2, fill = fillColor )
        canvasVisual.update()


root = tkinter.Tk()
k_gui = KmeansGui(root)
root.mainloop()
